@@ -34,9 +34,13 @@ config = load_configuration()
 
 def running_in_test_mode() -> bool:
     # A function to detect whether or not the system is running with Pytest
-    if ("pytest" in sys.modules) or ("PYTEST_CURRENT_TEST" in os.environ):
+    if "PYTEST_CURRENT_TEST" in os.environ:
         return True
     elif any(re.findall(r"pytest|py.test", sys.argv[0])):
         return True
     else:
         return False
+
+
+# list of environment variables for tricks
+ENV_VAR_TRACK_MISSING_DATA_FIELD = "PREHEAT_OPEN_TRACK_MISSING_FIELDS"
