@@ -5,42 +5,52 @@ import json
 import os
 from datetime import datetime
 from io import StringIO
-from typing import Dict, Optional
+from typing import Any, Optional, Union
 
 import numpy as np
 import pandas as pd
 from dateutil.tz import gettz
 from requests import Session
+from requests.adapters import HTTPAdapter
 from requests.models import Response
+from urllib3.util.retry import Retry
 
 import preheat_open
 
-from .logging import Logging
+from .exceptions import (  # For backwards compatibility
+    AccessDeniedError,
+    APIDataExtractError,
+    APIKeyMissingError,
+    MissingConfigurationFile,
+    MissingConfigurationFileError,
+)
+from .logging import Logging, write_log_to_profiling_file
 from .singleton import Singleton
 
-BASE_URL = "https://api.neogrid.dk/public/api/v1"
-MAX_CIDS_PER_REQ = 3
-MAX_POINTS_PER_REQ = 90000
+BASE_URL: str = "https://api.neogrid.dk/public/api/v1"
+MAX_CIDS_PER_REQ: int = 3
+MAX_POINTS_PER_REQ: int = 90000
+MAX_IDS_AND_CIDS_PER_REQUEST: int = 100
 
-GET_TIMEOUT_SECONDS = 60
-PUT_TIMEOUT_SECONDS = 10
+GET_TIMEOUT_SECONDS: int = 60
+PUT_TIMEOUT_SECONDS: int = 10
 
 # Perhaps we can steal this from the OS?
 TIMEZONE = gettz("Europe/Copenhagen")
-DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"
+DATETIME_FORMAT: str = "%Y-%m-%dT%H:%M:%S.%fZ"
 
-USER_LEVEL_CONFIGURATION_FILE = os.path.expanduser("~/.preheat/config.json")
-MACHINE_LEVEL_CONFIGURATION_FILE = os.path.expanduser("/etc/preheat/config.json")
-MACHINE_LEVEL_CONFIGURATION_FILE_OPT = os.path.expanduser(
+USER_LEVEL_CONFIGURATION_FILE: str = os.path.expanduser("~/.preheat/config.json")
+MACHINE_LEVEL_CONFIGURATION_FILE: str = os.path.expanduser("/etc/preheat/config.json")
+MACHINE_LEVEL_CONFIGURATION_FILE_OPT: str = os.path.expanduser(
     "/etc/opt/preheat/config.json"
 )
 
 
 def configuration_file_path() -> str:
     """
+    Determines the path of the configuration file to be used
 
-    :return:
-    :rtype:
+    :return: full path of the configuration file
     """
     if os.path.exists(USER_LEVEL_CONFIGURATION_FILE):
         out = USER_LEVEL_CONFIGURATION_FILE
@@ -49,58 +59,37 @@ def configuration_file_path() -> str:
     elif os.path.exists(MACHINE_LEVEL_CONFIGURATION_FILE):
         out = MACHINE_LEVEL_CONFIGURATION_FILE
     else:
-        raise MissingConfigurationFile()
+        raise MissingConfigurationFileError()
     return out
 
 
-class APIKeyMissingError(Exception):
-    pass
-
-
-class AccessDeniedError(Exception):
-    pass
-
-
-class APIDataExtractError(Exception):
-    pass
-
-
-class MissingConfigurationFile(Exception):
-    pass
-
-
 def api_string_to_datetime(t: str) -> datetime:
     """
+    Converts a datetime string from the API to a python datetime
 
-    :param t:
-    :type t:
-    :return:
-    :rtype:
+    :param t: datetime in string format
+    :return: python datetime
     """
     return datetime.strptime(t, "%Y-%m-%dT%H:%M:%S.%f%z")
 
 
 def datetime_to_api_string(t: datetime) -> str:
     """
+    Converts a python datetime to a string ready for use in the API I/O
 
-    :param t:
-    :type t:
+    :param t: python datetime
     :return:
-    :rtype:
     """
     return t.isoformat()
 
 
-def ids_list_to_string(list2use, separator: str = ",") -> str:
-    """Helper function to turn list into string, e.g. comma separated (default).
-
-    :param list2use:
-    :type list2use:
-    :param separator:
-    :type separator:
-    :return:
-    :rtype:
+def ids_list_to_string(list2use: list, separator: str = ",") -> str:
+    """
+    Helper function to turn list into string, e.g. comma separated (default).
 
+    :param list2use: list to convert
+    :param separator: separation character (default: ',')
+    :return: list in string format
     """
 
     if isinstance(list2use, list):
@@ -115,11 +104,11 @@ def ids_list_to_string(list2use, separator: str = ",") -> str:
     return res
 
 
-def load_configuration() -> Dict:
+def load_configuration() -> Union[None, dict]:
     """
+    Loads the API configuration
 
-    :return:
-    :rtype:
+    :return: None if no configuration file found, else a dictionary with the keys/values in the configuration file
     """
     config = None
     try:
@@ -129,47 +118,56 @@ def load_configuration() -> Dict:
             Logging().info(f"Loading config from {config_file}")
             with open(config_file, "r") as fp:
                 config = json.load(fp)
-    except:
+    except Exception:
         config = None
 
     return config
 
 
 class ApiSession(metaclass=Singleton):
-    """Classs for something about Apisessions???"""
+    """Singleton class to handle API connection sessions"""
 
     def __init__(self):
-        self.base_url = None
-        self.session = Session()
+
+        # Adding protection against remote closing connection
+        retry_strategy = Retry(
+            total=3,
+            status_forcelist=[429, 500, 502, 503, 504],
+            allowed_methods=["GET", "PUT", "POST"],
+        )
+        adapter = HTTPAdapter(max_retries=retry_strategy)
+        http_session = Session()
+        http_session.mount("https://", adapter)
+        http_session.mount("http://", adapter)
+
+        self.base_url = BASE_URL
+        self.session = http_session
         Logging().debug(f"Creating API session [{self.addr()}]...")
 
-        self.set_api_url()
         self.set_api_key()
 
-    def addr(self):
+    def addr(self) -> str:
         """
+        Making a representation of the session object for use in logging
 
-        :return:
-        :rtype:
+        :return: string representing the session
         """
         return hex(id(self.session))
 
     def get_api_key(self) -> str:
         """
+        Getter for the API key of the user
 
-        :return:
-        :rtype:
+        :return: the API key for the user
         """
-
         return self.session.headers.get("Authorization")
 
     def set_api_key(self, api_key: Optional[str] = None) -> None:
         """
+        Sets the API key
 
-        :param api_key:
-        :type api_key:
-        :return:
-        :rtype:
+        :param api_key: value of the API key (if None, loads from the configuration file)
+        :return: /
         """
         if api_key is None:
             api_key = self._load_api_key(
@@ -184,39 +182,42 @@ class ApiSession(metaclass=Singleton):
 
     def set_api_url(self, api_url=None):
         """
-
-        :param api_url:
-        :type api_url:
-        :return:
-        :rtype:
+        DEPRECATED (only kept for backwards-compatibility)
         """
-        if api_url is None:
-            api_url = self._load_api_url()
-        else:
-            Logging().debug(f"API url set manually [{self.addr()}]")
-        self.base_url = api_url
+        Logging().warning(
+            DeprecationWarning(f"Manual update of the API url is deprecated")
+        )
 
-    def api_get(self, endpoint, out="json", payload=None, headers=None):
+    def __log_api_io(self, msg: str) -> None:
+        Logging().debug(msg)
+        write_log_to_profiling_file("API-I/O: " + msg)
+
+    def api_get(
+        self,
+        endpoint: str,
+        out: str = "json",
+        payload: Optional[dict[str, Any]] = None,
+        headers: Optional[dict[str, Any]] = None,
+        bare_url: bool = False,
+    ):
         """
+        Carries out a get request on the API
 
-        :param endpoint:
-        :type endpoint:
-        :param out:
-        :type out:
+        :param endpoint: endpoint to call on the API
+        :param out: format for the output (json or csv)
         :param payload:
-        :type payload:
         :param headers:
-        :type headers:
+        :param bare_url:
         :return:
-        :rtype:
         """
         payload = payload if payload is not None else {}
         headers = headers if headers is not None else {}
+        self.__log_api_io(f"GET /{endpoint} [{self.addr()}]")
 
-        Logging().debug(f"GET /{endpoint} [{self.addr()}]")
-        # Logging().debug(f"{payload}")
-
-        path = f"{self.base_url}/{endpoint}"
+        if bare_url:
+            path = endpoint
+        else:
+            path = self.base_url + "/" + endpoint
 
         if out == "csv":
             headers["Accept"] = "text/csv"
@@ -241,27 +242,37 @@ class ApiSession(metaclass=Singleton):
             if out == "json":
                 return resp.json()
             elif out == "csv":
-                # Logging().debug(f"{resp.text}")
                 data = StringIO(resp.text)
                 return pd.read_csv(data, delimiter=";")
+            else:
+                raise ValueError("Invalid value for out parameter ({})".format(out))
+        except ValueError as e:
+            raise e
         except Exception as e:
             raise APIDataExtractError() from e
 
-    def api_put(self, endpoint, json_payload=None) -> Response:
+    def api_put(
+        self,
+        endpoint: str,
+        out: str = "json",
+        json_payload: Optional[dict[str, Any]] = None,
+        headers: Optional[dict[str, Any]] = None,
+        bare_url: bool = False,
+    ) -> Response:
         """
 
         :param endpoint:
-        :type endpoint:
         :param json_payload:
-        :type json_payload:
+        :param bare_url:
         :return:
-        :rtype:
         """
         json_payload = json_payload if json_payload is not None else {}
+        self.__log_api_io(f"PUT /{endpoint} [{self.addr()}]")
 
-        Logging().info(f"PUT /{endpoint} [{self.addr()}]")
-
-        path = f"{self.base_url}/{endpoint}"
+        if bare_url:
+            path = endpoint
+        else:
+            path = self.base_url + "/" + endpoint
         r = self.session.put(path, json=json_payload, timeout=PUT_TIMEOUT_SECONDS)
         try:
             r.raise_for_status()
@@ -276,16 +287,49 @@ class ApiSession(metaclass=Singleton):
             raise e
         return r
 
-    def _load_api_key(self, existing):
-        """Load API key from environment variable
+    def api_post(
+        self,
+        endpoint: str,
+        out: str = "json",
+        json_payload: Optional[dict[str, Any]] = None,
+        bare_url: bool = False,
+    ) -> Response:
+        """
+
+        :param endpoint:
+        :param json_payload:
+        :param bare_url:
+        :return:
+        """
+        json_payload = json_payload if json_payload is not None else {}
+        self.__log_api_io(f"POST /{endpoint} [{self.addr()}]")
+
+        if bare_url:
+            path = endpoint
+        else:
+            path = self.base_url + "/" + endpoint
+        r = self.session.post(path, json=json_payload, timeout=PUT_TIMEOUT_SECONDS)
+        try:
+            r.raise_for_status()
+        except Exception as e:
+            msg = """POST - FAILED
+            URL: {}
+            Payload: {}
+            """.format(
+                path, str(json_payload)
+            )
+            Logging().error(msg, exception=e)
+            raise e
+        return r
+
+    def _load_api_key(self, existing: Optional[str]) -> str:
+        """
+        Load API key from environment variable
         - if API key not present in environment variable it will be set None
         - will then break when trying to access data on a building
 
-        :param existing:
-        :type existing:
-        :return:
-        :rtype:
-
+        :param existing: the existing API key (None if not set)
+        :return: the value of the API key
         """
 
         # First respect any override by set_api_key
@@ -334,11 +378,11 @@ class ApiSession(metaclass=Singleton):
             else:
                 return str(None)
 
-    def _load_api_url(self):
+    def _load_api_url(self) -> str:
         """
+        Loads the API url
 
-        :return:
-        :rtype:
+        :return: the base URL of the API
         """
         if os.environ.get("PREHEAT_API_URL_OVERRIDE") is not None:
             url = os.environ.get("PREHEAT_API_URL_OVERRIDE")
@@ -349,13 +393,12 @@ class ApiSession(metaclass=Singleton):
             return BASE_URL
 
 
-def set_api_key(api_key: str) -> None:
+def set_api_key(api_key: Optional[str]) -> None:
     """
+    Sets the API key
 
-    :param api_key:
-    :type api_key:
-    :return:
-    :rtype:
+    :param api_key: value of the API key
+    :return: /
     """
     if api_key is None:
         if "PREHEAT_API_KEY_OVERRIDE" in os.environ:
@@ -367,49 +410,77 @@ def set_api_key(api_key: str) -> None:
     ApiSession().set_api_key()
 
 
+def api_get(
+    endpoint: str,
+    out: str = "json",
+    payload: Optional[dict[str, Any]] = None,
+    headers: Optional[dict[str, Any]] = None,
+    bare_url: bool = False,
+) -> Response:
+    """
+
+    :param endpoint:
+    :param out:
+    :param payload:
+    :param headers:
+    :return:
+    """
+    return ApiSession().api_get(
+        endpoint,
+        out=out,
+        payload=payload,
+        headers=headers,
+        bare_url=bare_url,
+    )
+
+
+def api_post(
+    endpoint: str,
+    out: str = "json",
+    json_payload: Optional[dict[str, Any]] = None,
+    bare_url: bool = False,
+) -> Response:
+    """
+
+    :param endpoint:
+    :param out:
+    :param json_payload:
+    :param headers:
+    :return:
+    """
+    return ApiSession().api_post(
+        endpoint,
+        out=out,
+        json_payload=json_payload,
+        bare_url=bare_url,
+    )
+
+
+def api_put(
+    endpoint: str, json_payload: Optional[dict[str, Any]] = None, bare_url: bool = False
+) -> Response:
+    """
+
+    :param endpoint:
+    :param json_payload:
+    :return:
+    """
+    return ApiSession().api_put(
+        endpoint,
+        out="json",
+        json_payload=json_payload,
+        bare_url=bare_url,
+    )
+
+
+# For backwards compatibility (deprecated methods and classes)
 def set_api_url(api_url: str) -> None:
     """
-
-    :param api_url:
-    :type api_url:
-    :return:
-    :rtype:
+    Only kept for backwards compatibility
     """
-    if api_url is None:
-        if "PREHEAT_API_URL_OVERRIDE" in os.environ:
-            os.environ.pop("PREHEAT_API_URL_OVERRIDE")
-            Logging().debug("API key override unset")
-    else:
-        os.environ["PREHEAT_API_URL_OVERRIDE"] = api_url
-        Logging().debug("API key override set")
-    ApiSession().set_api_url()
+    Logging().warning(DeprecationWarning(f"Manual update of the API url is deprecated"))
+    ApiSession().set_api_url(api_url)
 
 
-def api_get(endpoint: str, out: str = "json", payload=None, headers=None) -> Response:
-    """
-
-    :param endpoint:
-    :type endpoint:
-    :param out:
-    :type out:
-    :param payload:
-    :type payload:
-    :param headers:
-    :type headers:
-    :return:
-    :rtype:
-    """
-    return ApiSession().api_get(endpoint, out, payload, headers)
-
-
-def api_put(endpoint: str, json_payload=None) -> Response:
-    """
-
-    :param endpoint:
-    :type endpoint:
-    :param json_payload:
-    :type json_payload:
-    :return:
-    :rtype:
-    """
-    return ApiSession().api_put(endpoint, json_payload)
+def __enforce_imports():
+    f = [APIKeyMissingError(), AccessDeniedError(), MissingConfigurationFile()]
