@@ -1,12 +1,18 @@
 from __future__ import annotations
 
 import re
-from typing import List, Optional, Union
+from typing import Optional, Union
 
 from .backwards_compatibility import load_parameter_old_naming
-from .helpers import convenience_result_list_shortener
+from .helpers import (
+    check_no_remaining_fields,
+    convenience_result_list_shortener,
+    sanitise_unit_type_input,
+)
 from .logging import Logging
+from .types import TYPE_DATETIME_INPUT
 from .unit import Unit
+from .weather import Weather
 
 
 def populate_units(unit_type, units_data, building_ref):
@@ -26,7 +32,7 @@ def add_unique_units(units, new_units, ids):
     return
 
 
-def exclude_shared_units_from_list(x: List[BaseBuildingUnit]) -> List[BaseBuildingUnit]:
+def exclude_shared_units_from_list(x: list[BaseBuildingUnit]) -> list[BaseBuildingUnit]:
     y = []
     for u in x:
         if not u.is_shared():
@@ -37,8 +43,10 @@ def exclude_shared_units_from_list(x: List[BaseBuildingUnit]) -> List[BaseBuildi
 class BaseBuildingUnit(Unit):
     """Building Unit; an extension of Unit to handle zones and parent units"""
 
-    def __init__(self, unit_type, unit_data, building_ref):
-        super().__init__(unit_type, unit_data)
+    def __init__(
+        self, unit_type: str, unit_data: dict, building_ref, load_data_by: str = "id"
+    ):
+        super().__init__(unit_type, unit_data, load_data_by=load_data_by)
 
         # Building reference
         self.building = building_ref
@@ -46,7 +54,13 @@ class BaseBuildingUnit(Unit):
         # Zones covered (Attempt to first extract 'zoneIds', then ['zoneId']), and couple zones
         self.zone_ids = []
         self.__zones = []
-        for zone_id in unit_data.pop("zoneIds", [unit_data.pop("zoneId", [])]):
+        zone_ids = unit_data.pop("zoneIds", None)
+        if zone_ids is None:
+            if "zoneId" in unit_data:
+                zone_ids = [unit_data.pop("zoneId", None)]
+            else:
+                zone_ids = []
+        for zone_id in zone_ids:
             self.__add_zone(zone_id)
         for z_i in self.__zones:
             z_i.add_coupled_unit(self)
@@ -58,12 +72,16 @@ class BaseBuildingUnit(Unit):
         self.zone_ids.append(zone_id)
         self.__zones += self.building.get_zones([zone_id])
 
-    def parents(self) -> List[BaseBuildingUnit]:
+    def parents(self) -> list[BaseBuildingUnit]:
         return [
             self.building.G.nodes[item[0]]["unit"]
             for item in self.building.G.pred[self.id].items()
         ]
 
+    @property
+    def weather(self) -> Weather:
+        return self.building.weather
+
     # Taken from open API: building_unit.py
     def query_parent_units(
         self,
@@ -71,7 +89,7 @@ class BaseBuildingUnit(Unit):
         name: Optional[str] = None,
         exclude_shared: bool = False,
         _depth: int = 5,
-    ) -> List[BaseBuildingUnit]:
+    ) -> list[BaseBuildingUnit]:
         # if no levels left to traverse no parents to output
         if _depth < 1 or type(_depth) is not int:
             return []
@@ -127,17 +145,17 @@ class BaseBuildingUnit(Unit):
 
         return result
 
-    def qpu(self, *args, **kwargs) -> Union[BaseBuildingUnit, List[BaseBuildingUnit]]:
+    def qpu(self, *args, **kwargs) -> Union[BaseBuildingUnit, list[BaseBuildingUnit]]:
         result = self.query_parent_units(*args, **kwargs)
         return convenience_result_list_shortener(result)
 
     def get_zones(self):
         return self.__zones
 
-    def query_units(self, **kwargs) -> List[BaseBuildingUnit]:
+    def query_units(self, **kwargs) -> list[BaseBuildingUnit]:
         return []
 
-    def qu(self, *args, **kwargs) -> Union[BaseBuildingUnit, List[BaseBuildingUnit]]:
+    def qu(self, *args, **kwargs) -> Union[BaseBuildingUnit, list[BaseBuildingUnit]]:
         result = self.query_units(*args, **kwargs)
         return convenience_result_list_shortener(result)
 
@@ -190,14 +208,22 @@ class BuildingUnit(BaseBuildingUnit):
         self._related_sub_units_refs += unit_data.pop("coolingCoilIds", [])
         # self.sub_unit_refs += unit_data.pop("zoneIds", [unit_data.pop("zoneId", [])])
         self._related_meters_refs = []
-        meter_labels = ["electricityId", "mainId", "heatingId", "hotWaterId"]
+        meter_labels = [
+            "electricityId",
+            "mainId",
+            "heatingId",
+            "hotWaterId",
+            "coldWaterId",
+        ]
         for label in meter_labels:
             if label in unit_data and unit_data[label]:
-                self._related_meters_refs.append(unit_data[label])
+                self._related_meters_refs.append(unit_data.pop(label))
 
         # For now, this is a protected attribute
         self._meters = []
 
+        check_no_remaining_fields(unit_data, debug_helper="building_unit_data")
+
     def add_sub_unit(self, u):
         if isinstance(u, list):
             [self.add_sub_unit(u_i) for u_i in u]
@@ -210,8 +236,8 @@ class BuildingUnit(BaseBuildingUnit):
 
     def load_data(
         self,
-        start,
-        end,
+        start: TYPE_DATETIME_INPUT,
+        end: TYPE_DATETIME_INPUT,
         resolution="minute",
         load_children=False,
         components=None,
@@ -230,13 +256,13 @@ class BuildingUnit(BaseBuildingUnit):
                     load_children=True,
                 )
 
-    def clear_data(self, clear_children=False) -> None:
+    def clear_data(self, clear_children: bool = False) -> None:
         super().clear_data()
         if clear_children is True:
             for u_i in self.children():
                 u_i.clear_data(clear_children=True)
 
-    def children(self) -> List[BaseBuildingUnit]:
+    def children(self) -> list[BaseBuildingUnit]:
         return [
             self.building.G.nodes[item[0]]["unit"]
             for item in self.building.G.succ[self.id].items()
@@ -248,13 +274,14 @@ class BuildingUnit(BaseBuildingUnit):
         name: Optional[str] = None,
         exclude_shared: bool = False,
         _depth: int = 10,
-    ) -> List[BaseBuildingUnit]:
+    ) -> list[BaseBuildingUnit]:
         # If we pass a unit_type, check if we do regex search or strict search
         if unit_type:
             if unit_type[0] == "?":
                 r_type = re.compile(unit_type[1:])
                 type_match = lambda t: r_type.search(t)
             else:
+                unit_type = sanitise_unit_type_input(unit_type)
                 type_match = lambda t: t == unit_type
 
         # If we pass name, check if we do regex search or strict search
