@@ -1,7 +1,10 @@
+from typing import Any
+
+
 class Component(object):
     """Defines a Component in the PreHEAT sense"""
 
-    def __init__(self, component_data):
+    def __init__(self, component_data: dict[str, Any]):
         # Identifier of the component
         self.cid = component_data.pop("cid", None)
         self.id = component_data.pop("id", None)
@@ -21,5 +24,5 @@ class Component(object):
         # Standard unit for this type of component
         self.std_unit = component_data.pop("stdUnit", "")
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "{0}({1}, {2})".format(type(self).__name__, self.name, self.tag)
