@@ -1,20 +1,34 @@
+"""
+To manage timeseries data loading
+"""
 import math
+from datetime import datetime, timedelta
 from functools import reduce
+from typing import Optional
 
 import numpy as np
 import pandas as pd
 
 from .api import (
     DATETIME_FORMAT,
+    MAX_IDS_AND_CIDS_PER_REQUEST,
     MAX_POINTS_PER_REQ,
-    TIMEZONE,
     APIDataExtractError,
     api_get,
+    datetime_to_api_string,
 )
 from .helpers import list_to_string, sanitise_datetime_input
+from .types import TYPE_DATETIME_INPUT
+
+__DEFAULT_DATAFRAME_INDEX = pd.DatetimeIndex([], tz="UTC")
 
 
-def load_box_data(component_map, start, end, resolution="minute") -> pd.DataFrame:
+def load_box_data(
+    component_map,
+    start: TYPE_DATETIME_INPUT,
+    end: TYPE_DATETIME_INPUT,
+    resolution="minute",
+) -> pd.DataFrame:
     # Parse strings to datetime object
 
     start = sanitise_datetime_input(start)
@@ -26,14 +40,19 @@ def load_box_data(component_map, start, end, resolution="minute") -> pd.DataFram
     cids = list(component_map.keys())
 
     if not cids:
-        return pd.DataFrame()
+        return pd.DataFrame(data={}, index=__DEFAULT_DATAFRAME_INDEX)
 
     resp = __perform_requests(path, cid_key, cids, start, end, resolution, "csv")
 
     return __extract_data(resp, "cid", component_map, "csv").astype(float)
 
 
-def load_model_data(component_map, start, end, resolution="raw") -> pd.DataFrame:
+def load_model_data(
+    component_map,
+    start: TYPE_DATETIME_INPUT,
+    end: TYPE_DATETIME_INPUT,
+    resolution="raw",
+) -> pd.DataFrame:
     # Parse strings to datetime object
     start = sanitise_datetime_input(start)
     end = sanitise_datetime_input(end)
@@ -44,7 +63,7 @@ def load_model_data(component_map, start, end, resolution="raw") -> pd.DataFrame
     ids = list(component_map.keys())
 
     if not ids or len(ids) == 0:
-        return pd.DataFrame()
+        return pd.DataFrame(data={}, index=__DEFAULT_DATAFRAME_INDEX)
 
     if resolution != "raw":
         # Do chunked requests
@@ -59,9 +78,9 @@ def load_model_data(component_map, start, end, resolution="raw") -> pd.DataFrame
 def load_weather_data(
     location_id,
     component_map,
-    start,
-    end,
-    resolution="hour",
+    start: TYPE_DATETIME_INPUT,
+    end: TYPE_DATETIME_INPUT,
+    resolution: str = "hour",
     components=None,
 ) -> pd.DataFrame:
     # Parse strings to datetime object
@@ -76,27 +95,45 @@ def load_weather_data(
     # If minutes, then we need to resample
     if resolution in ["minute", "5min", "raw"]:
         resolution = "hour"
+        t_span = (start - timedelta(hours=1), end + timedelta(hours=1), resolution)
         resample = True
     else:
+        t_span = (start, end, resolution)
         resample = False
 
-    resp = __perform_requests(path, cid_key, cids, start, end, resolution, "csv")
+    resp = __perform_requests(path, cid_key, cids, *t_span, "csv")
 
     if resample:
         return (
             __extract_data(resp, "type_id", component_map, "csv")
             .resample("5min")
-            .ffill()
+            .ffill()[start:end]
         )
 
     return __extract_data(resp, "type_id", component_map, "csv").astype(float)
 
 
-def perform_requests(path, cid_key, cids, start, end, resolution, format="csv"):
+def perform_requests(
+    path,
+    cid_key,
+    cids,
+    start: TYPE_DATETIME_INPUT,
+    end: TYPE_DATETIME_INPUT,
+    resolution: str,
+    format: str = "csv",
+):
     return __perform_requests(path, cid_key, cids, start, end, resolution, format)
 
 
-def __perform_requests(path, cid_key, cids, start, end, resolution, format="csv"):
+def __perform_requests(
+    path,
+    cid_key,
+    cids,
+    start: TYPE_DATETIME_INPUT,
+    end: TYPE_DATETIME_INPUT,
+    resolution: str,
+    format: str = "csv",
+):
     # Split CIDs into chunks
     # NOTE: we reduce number of CIDs pr. REQ to confine w. MAX_POINTS_PER_REQ
     # This sets the limit to 90000 points on a single CID; which for 5min.
@@ -124,6 +161,9 @@ def __perform_requests(path, cid_key, cids, start, end, resolution, format="csv"
     else:
         cid_chunk_size = (MAX_POINTS_PER_REQ / N_points_requested) * len(cids)
 
+        # Imposing another max of number of IDs and CIDs regardless of resolution
+        cid_chunk_size = min(cid_chunk_size, MAX_IDS_AND_CIDS_PER_REQUEST)
+
         cid_chunks = [
             cids[x : x + max(int(cid_chunk_size), 1)]
             for x in range(0, len(cids), max(int(cid_chunk_size), 1))
@@ -148,9 +188,11 @@ def __perform_requests(path, cid_key, cids, start, end, resolution, format="csv"
     return reqs
 
 
-def __make_request(path, payload, start, end, resolution, format="csv"):
-    payload["start_time"] = start.isoformat()
-    payload["end_time"] = end.isoformat()
+def __make_request(
+    path, payload, start: datetime, end: datetime, resolution: str, format: str = "csv"
+):
+    payload["start_time"] = datetime_to_api_string(start)
+    payload["end_time"] = datetime_to_api_string(end)
 
     if resolution == "5min":
         resolution = "minute"
@@ -164,14 +206,54 @@ def extract_data(resp, id_key, component_map, format="csv"):
     return __extract_data(resp, id_key, component_map, format)
 
 
-def __extract_data(resp, id_key, component_map, format="csv"):
+def __reformat_dataframe(
+    resp: pd.DataFrame,
+    component_map: dict[str, str],
+    id_key: Optional[str] = None,
+) -> pd.DataFrame:
 
     kwargs_to_datetime = dict(utc=True, format=DATETIME_FORMAT)
     # If this is having an issue, use "infer_datetime_format=True" instead of "format=..."
 
+    # Use of resp.loc["time"] was removed below as it triggers warnings for pandas>=1.5
+    resp["time"] = pd.to_datetime(resp["time"], **kwargs_to_datetime)
+
+    resp_columns = resp.keys()
+    if id_key is not None:
+        components = id_key
+    elif "id" in resp_columns:
+        components = "id"
+    elif "type_id" in resp_columns:
+        components = "type_id"
+    else:
+        raise RuntimeError("Unknown column formatting")
+    comp_map_rename = {int(i): component_map[i] for i in component_map}
+    component_names = comp_map_rename.values()
+    if resp.empty:
+        # For backwards compatibility
+        out = pd.DataFrame(
+            {i: [] for i in component_names}, index=__DEFAULT_DATAFRAME_INDEX
+        )
+    else:
+        out = resp.pivot(index="time", columns=components, values="value").rename(
+            columns=comp_map_rename,
+        )
+
+    for c in component_names:
+        if c not in out.keys():
+            out[c] = np.nan
+
+    out.axes[1].name = None  # For backwards compatibility
+    return out
+
+
+def __extract_data(resp, id_key, component_map, format="csv"):
+
     if len(resp) < 1:
         # Short circuit all in case of empty response
-        return pd.DataFrame()
+        out = pd.DataFrame(
+            {i: [] for i in component_map.values()}, index=__DEFAULT_DATAFRAME_INDEX
+        )
     elif format == "json":
         if isinstance(resp, list):
             # Initialize empty response that we can append to
@@ -184,6 +266,7 @@ def __extract_data(resp, id_key, component_map, format="csv"):
         if "message" in resp:
             raise APIDataExtractError(resp["message"])
 
+        kwargs_to_datetime = dict(utc=True, format=DATETIME_FORMAT)
         dfs = []
         for cid, name in component_map.items():
             time = pd.to_datetime(
@@ -194,6 +277,11 @@ def __extract_data(resp, id_key, component_map, format="csv"):
             df_i = pd.DataFrame(value, columns=[name], index=time)
             dfs.append(df_i)
 
+        out = reduce(
+            lambda x, y: pd.merge(x, y, left_index=True, right_index=True, how="outer"),
+            dfs,
+        )
+
     elif format == "csv":
         # Join response into one
         if isinstance(resp, list):
@@ -202,38 +290,18 @@ def __extract_data(resp, id_key, component_map, format="csv"):
                 resp_new = df if resp_new is None else pd.concat([resp_new, df])
             resp = resp_new
 
-        # Split
-        dfs = []
-        for cid, name in component_map.items():
-            df_i = resp[(resp[id_key] == int(cid))].copy()
-            df_i.loc[:, "time"] = pd.to_datetime(df_i["time"], **kwargs_to_datetime)
-            df_i = df_i.set_index("time")
-            df_i = df_i.filter(items=["value"])
-            df_i = df_i.rename(columns={"value": name})
-            df_i.index = df_i.index.rename(None)
-            dfs.append(df_i)
+        out = __reformat_dataframe(resp, component_map, id_key)
 
     elif format == "dataframe":
-        # Split
-        dfs = []
-        for cid, name in component_map.items():
-            df_i = resp[(resp[id_key] == int(cid))].copy()
-            df_i.loc[:, "time"] = pd.to_datetime(df_i["time"], **kwargs_to_datetime)
-            df_i = df_i.set_index("time")
-            df_i = df_i.filter(items=["value"])
-            df_i = df_i.rename(columns={"value": name})
-            df_i.index = df_i.index.rename(None)
-            dfs.append(df_i)
+        out = __reformat_dataframe(resp, component_map, id_key)
 
     else:
         raise TypeError("Unsupported format")
 
-    df = reduce(
-        lambda x, y: pd.merge(x, y, left_index=True, right_index=True, how="outer"), dfs
-    )
-    return df
+    out.index.rename(None, inplace=True)
+    return out
 
 
-def __check_start_end_date(start, end):
+def __check_start_end_date(start: datetime, end: datetime) -> None:
     if end < start:
         raise ValueError("End date must be AFTER start date")
