@@ -1,12 +1,15 @@
 import warnings
 from datetime import timezone
+from typing import Any
 
 import numpy as np
 import pandas as pd
 from dateutil.tz import gettz
 
 from .api import api_get, api_string_to_datetime
-from .helpers import list_to_string, timestep_start
+from .backwards_compatibility import load_parameter_old_naming
+from .helpers import list_to_string, sanitise_datetime_input, timestep_start
+from .types import TYPE_DATETIME_INPUT
 
 utc = timezone.utc
 
@@ -38,7 +41,18 @@ class SupplyPoint:
     def get_all_price_component_ids(self):
         return [pc_i.price_component.id for pc_i in self.get_price_components()]
 
-    def load_price_data(self, start_time, end_time):
+    def load_price_data(
+        self,
+        start: TYPE_DATETIME_INPUT = None,
+        end: TYPE_DATETIME_INPUT = None,
+        **kwargs,
+    ):
+
+        start, end, __ = load_parameter_old_naming(
+            start, end, resolution="", postfix="time", **kwargs
+        )
+        start_time = sanitise_datetime_input(start)
+        end_time = sanitise_datetime_input(end)
         url_postfix = "pricecomponents/prices"
         payload = {
             "ids": list_to_string(self.get_all_price_component_ids()).replace(" ", ""),
@@ -66,7 +80,11 @@ class PriceDataCollection:
         self.__applied_price_components = applied_price_components
         self.__data = None
 
-    def load_data(self, price_component_data, start, end):
+    def load_data(
+        self, price_component_data, start: TYPE_DATETIME_INPUT, end: TYPE_DATETIME_INPUT
+    ):
+        start = sanitise_datetime_input(start)
+        end = sanitise_datetime_input(end)
         tz_out = utc
         start_hour = timestep_start("hour", t=start).astimezone(tz_out)
         end_hour = timestep_start("hour", t=end).astimezone(tz_out)
@@ -109,7 +127,7 @@ class PriceDataCollection:
                                     row_ij["validFrom"],
                                     row_ij["validTo"],
                                     freq="H",
-                                    closed="left",
+                                    inclusive="left",
                                 )
                             )
                             df_base_ij["hour"] = df_base_ij.index.tz_convert(
@@ -140,21 +158,15 @@ class PriceDataCollection:
                                     row_ij["validFrom"],
                                     row_ij["validTo"],
                                     freq=freq2use,
-                                    closed="left",
+                                    inclusive="left",
                                 )
                             )
-
-                            dt_hours = (
-                                pd.Timestamp(row_ij["validTo"])
-                                - pd.Timestamp(row_ij["validFrom"])
-                            ).total_seconds() / 3600
-                            fill_method = "ffill"
-                            df_i["price"] = row_ij["price"][0] / dt_hours
+                            df_i["price"] = row_ij["price"][0]
 
                         if df_price_raw_i is None:
-                            df_price_raw_i = df_i.copy()
+                            df_price_raw_i = df_i.ffill()
                         else:
-                            df_price_raw_i = df_price_raw_i.append(df_i.copy())
+                            df_price_raw_i = df_price_raw_i.append(df_i.ffill())
 
                 df_price_i = df_price_raw_i[["price"]]
 
@@ -255,7 +267,7 @@ class PriceDataCollection:
 
 
 class AppliedPriceComponent:
-    def __init__(self, data):
+    def __init__(self, data: dict[str, Any]):
         self.id = data["id"]
         self.supplyPointId = data["supplyPointId"]
         self.price_component = PriceComponent(data["priceComponent"])
@@ -276,7 +288,7 @@ class AppliedPriceComponent:
 
 
 class PriceComponent:
-    def __init__(self, data):
+    def __init__(self, data: dict[str, Any]):
         self.id = data["id"]
         self.name = data["name"]
         self.description = data["description"]
@@ -307,13 +319,13 @@ class PriceComponent:
 
 
 class Authority:
-    def __init__(self, data):
+    def __init__(self, data: dict[str, Any]):
         self.id = data["id"]
         self.name = data["name"]
 
 
 class PriceArea:
-    def __init__(self, data):
+    def __init__(self, data: dict[str, Any]):
         self.id = data["id"]
         self.name = data["name"]
         self.supply_type = data["supplyType"]
