@@ -1,12 +1,16 @@
+from copy import deepcopy
+
 import pandas as pd
 
 from preheat_open import test
+from preheat_open.weather import Weather
 
 
 class TestBuilding(test.PreheatTest):
     def test_building(self, building):
         assert building is not None
         assert building.weather is not None
+        assert isinstance(building.weather, Weather)
         assert building.units is not None
         assert len(building.units) > 0
         assert building.units["heating"] is not None
@@ -44,17 +48,18 @@ class TestBuilding(test.PreheatTest):
         assert not building_with_data.weather.data.empty
 
         # Testing selective data loading
-        building_with_data.clear_data()
+        new_building_with_data = deepcopy(building_with_data)
+        new_building_with_data.clear_data()
         comps = {
             "weather": ["Temperature", "Humidity"],
             "main": None,
             "secondaries": ["supplyT"],
         }
-        building_with_data.load_data(*short_period, components=comps)
+        new_building_with_data.load_data(*short_period, components=comps)
 
-        df_weather = building_with_data.weather.data
-        df_main = building_with_data.query_units("main")[0].data
-        df_secondary = building_with_data.query_units("secondaries")[0].data
+        df_weather = new_building_with_data.weather.data
+        df_main = new_building_with_data.query_units("main")[0].data
+        df_secondary = new_building_with_data.query_units("secondaries")[0].data
 
         assert df_weather.empty is False
         assert "Temperature" in df_weather
@@ -67,17 +72,30 @@ class TestBuilding(test.PreheatTest):
         assert "supplyT" in df_secondary
         assert "returnT" not in df_secondary
 
-        assert building_with_data.query_units("indoorClimate")[0].data.empty is True
-        assert building_with_data.query_units("heating")[0].data.empty is True
+        assert new_building_with_data.query_units("indoorClimate")[0].data.empty is True
+        assert new_building_with_data.query_units("heating")[0].data.empty is True
 
     def test_get_sub_zones(self, building_with_data):
         assert building_with_data.get_zones([]) == []
         assert len(building_with_data.get_zones([1])) == 0
         assert len(building_with_data.get_zones([4830])) == 1
         assert len(building_with_data.get_zones([4830, 4834])) == 2
+        assert len(building_with_data.query_zones(zone_id=0)) == 0
+        assert len(building_with_data.query_zones(zone_id=4830)) == 1
+        assert len(building_with_data.query_zones(zone_type="apartment")) >= 1
+        assert (
+            len(building_with_data.query_zones(zone_id=4830, zone_type="apartment"))
+            == 1
+        )
+        assert (
+            len(building_with_data.query_zones(zone_id=4830, zone_type="doesnt exist"))
+            == 0
+        )
+        assert len(building_with_data.query_zones(zone_type="doesnt exist")) == 0
 
-    def test_describe(self, building):
-        assert isinstance(building.describe(display=False), str)
+    def test_describe(self, building_with_devices):
+        assert isinstance(building_with_devices.describe(display=False), str)
+        assert isinstance(building_with_devices.describe_devices(display=False), str)
 
 
 def test_available_buildings():
@@ -85,3 +103,4 @@ def test_available_buildings():
 
     df = available_buildings()
     assert isinstance(df, pd.DataFrame)
+    assert len(df) == 1, "The API key has access to too many buildings"
