@@ -1,4 +1,7 @@
+from copy import deepcopy
+
 from preheat_open import test
+from preheat_open.weather import Weather
 
 
 class TestBuildingUnit(test.PreheatTest):
@@ -6,6 +9,7 @@ class TestBuildingUnit(test.PreheatTest):
         assert unit_with_data.is_shared() is False
         assert len(unit_with_data.query_units("missing")) == 0
         assert len(unit_with_data.building.query_units("main", exclude_shared=True)) > 0
+        assert isinstance(unit_with_data.weather, Weather)
 
     def test_load_data(self, unit_with_data):
         assert unit_with_data is not None
@@ -19,13 +23,15 @@ class TestBuildingUnit(test.PreheatTest):
         assert True is True
 
     def test_clear_data(self, unit_with_data):
-        unit_with_data.clear_data()
-        assert unit_with_data.data is not None
-        assert unit_with_data.data.empty
+        new_unit_with_data = deepcopy(unit_with_data)
+        new_unit_with_data.clear_data()
+        assert new_unit_with_data.data is not None
+        assert new_unit_with_data.data.empty
 
-        unit_with_data.clear_data(clear_children=True)
-        assert unit_with_data.data is not None
-        assert unit_with_data.data.empty
+        new_unit_with_data2 = deepcopy(unit_with_data)
+        new_unit_with_data2.clear_data(clear_children=True)
+        assert new_unit_with_data2.data is not None
+        assert new_unit_with_data2.data.empty
 
     def test_get_zones(self, building_with_data):
         from preheat_open.zone import Zone
