@@ -1,4 +1,5 @@
-import pandas as pd
+from copy import deepcopy
+
 import pytest
 
 from preheat_open import test
@@ -17,9 +18,10 @@ class TestBuilding(test.PreheatTest):
             assert not unit_with_data.data.empty
 
     def test_clear_data(self, unit_with_data):
-        unit_with_data.clear_data()
-        assert unit_with_data.data is not None
-        assert unit_with_data.data.empty
+        unit_with_data_to_clear = deepcopy(unit_with_data)
+        unit_with_data_to_clear.clear_data()
+        assert unit_with_data_to_clear.data is not None
+        assert unit_with_data_to_clear.data.empty
 
     def test_selected_data_loading(self, unit, short_period):
         unit.load_data(*short_period, components=["energy", "power"])
@@ -33,7 +35,7 @@ class TestBuilding(test.PreheatTest):
     def test_get_state(self, unit, short_period):
         s = unit.get_state(update=True, estimate="last", seconds_back=300)
         # The location does not get new data
-        assert len(s) == 0
+        assert s.isnull().all()
         # retrieve old state
         t_now = short_period[0]
         s = unit.get_state(update=True, estimate="last", seconds_back=300, t_now=t_now)
